var express = require('express');
var http = require('http');
var path = require('path');
var favicon = require('static-favicon');
var logger = require('morgan');
var cookieParser = require('cookie-parser');
var bodyParser = require('body-parser');
var uuid = require('node-uuid');

var routes = require('./routes');
var login = require('./routes/login');
var project = require('./routes/project');

var app = express();

/**
 * ログインチェック用の関数
 */
var loginCheck = function(req, res, next) {
    if(req.session.user){
      next();
    }else{
      res.redirect('/login');
    }
};

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'jade');

app.use(favicon());
app.use(logger('dev'));
app.use(bodyParser.json());
app.use(bodyParser.urlencoded());
app.use(cookieParser());
app.use(express.session({secret: uuid.v4()}));
app.use(express.static(path.join(__dirname, 'public')));
app.use(app.router);


app.get('/', loginCheck, routes.index);
app.post('/tree', loginCheck, routes.tree);
app.post('/tree_children', loginCheck, routes.tree_children);
app.post('/runJasmine', loginCheck, routes.runJasmine);


app.get('/project', loginCheck, project.index);
app.post('/project/add', loginCheck, project.add);
app.post('/project/delete', loginCheck, project.delete);
app.post('/project/update', loginCheck, project.update);

app.get('/login', login.login);
app.get('/register', login.register);
app.get('/logout', function(req, res){
  req.session.destroy();
  console.log('deleted sesstion');
  res.redirect('/');
});
app.post('/login/doLogin', login.doLogin);
app.post('/login/doRegister', login.doRegister);


/// catch 404 and forwarding to error handler
app.use(function(req, res, next) {
    var err = new Error('Not Found');
    err.status = 404;
    next(err);
});

/// error handlers

// development error handler
// will print stacktrace
if (app.get('env') === 'development') {
    app.use(function(err, req, res, next) {
        res.render('error', {
            message: err.message,
            error: err
        });
    });
}

// production error handler
// no stacktraces leaked to user
app.use(function(err, req, res, next) {
    res.render('error', {
        message: err.message,
        error: {}
    });
});


module.exports = app;
