var UiUtil = (function() {
  /**
   * FormのSubmit処理
   * @param {function} beforeCallback 送信前のコールバック 戻り値がfalseの場合はキャンセルする
   * @param {function} completeCallback 送信後のコールバック
   */
  function _submitForm(id, beforeCallback ,completeCallback) {
    $(id).submit(function(event) {
      event.preventDefault();
      var $form = $(this);
      var $button = $form.find('button');
      if (beforeCallback) {
        if (!beforeCallback($form)) {
          return;
        }
      }
      $.ajax({
        url: $form.attr('action'),
        type: $form.attr('method'),
        data: $form.serialize(),
        cache : false,
        timeout: 10000,

        // 送信前
        beforeSend: function(xhr, settings) {
          // ボタンを無効化し、二重送信を防止
          $button.attr('disabled', true);
        },
        // 応答後
        complete: function(xhr, textStatus) {
          // ボタンを有効化し、再送信を許可
          $button.attr('disabled', false);
        },
        
        // 通信成功時の処理
        success: function(result, textStatus, xhr) {
          // 入力値を初期化
          //$form[0].reset();
          completeCallback(null, result);
        },
        
        // 通信失敗時の処理
        error: function(xhr, textStatus, error) {
          if (error) {
            completeCallback(error, null);
          } else {
            completeCallback('ConnectError', null);
          }
        }
      });
    });
  }

  /**
   * メッセージの取得
   */
  function _getMessage(id) {
    return $('#message').find('#' + id).text();
  }

  var formatRegExp = /%[sdj%]/g;
  function _format(f) {
    if (!_isString(f)) {
      var objects = [];
      for (var i = 0; i < arguments.length; i++) {
        objects.push(arguments[i]);
      }
      return objects.join(' ');
    }

    var i = 1;
    var args = arguments;
    var len = args.length;
    var str = String(f).replace(formatRegExp, function(x) {
      if (x === '%%') return '%';
      if (i >= len) return x;
      switch (x) {
        case '%s': return String(args[i++]);
        case '%d': return Number(args[i++]);
        case '%j':
          try {
            return JSON.stringify(args[i++]);
          } catch (_) {
            return '[Circular]';
          }
        default:
          return x;
      }
    });
    for (var x = args[i]; i < len; x = args[++i]) {
      if (isNull(x) || !isObject(x)) {
        str += ' ' + x;
      } else {
        str += ' ' + inspect(x);
      }
    }
    return str;
  }

  function _isObject(arg) {
    return typeof arg === 'object' && arg !== null;
  }

  function _isNull(arg) {
    return arg === null;
  }

  function _isString(arg) {
    return typeof arg === 'string';
  }

  function _isUndefined(arg) {
    return arg === void 0;
  }

  function _htmlescape(str) {
    return String(str)
             .replace(/&/g, '&amp;')
             .replace(/"/g, '&quot;')
             .replace(/</g, '&lt;')
             .replace(/>/g, '&gt')
             .replace(/\n/g, ' <BR>\n ');
  }

  return {
    submitForm: _submitForm,
    getMessage: _getMessage,
    format: _format,
    isObject: _isObject,
    isNull: _isNull,
    isString: _isString,
    isUndefined: _isUndefined,
    htmlEscape: _htmlescape
  };
})();
