var db = require('../models');
var message = require('./message.js');
var async = require('async');
var dbUtil = require('./dbUtil.js');
var fs = require('fs');
var path = require('path');
var exec = require('child_process').exec;
var util = require('util');
var mkdirp = require('mkdirp');
var to_json = require('xmljson').to_json;


exports.index = function(req, res) {
  db.Project.findAll({ include: [{
    model:db.RootPath
  }]}).success(function(result) {
    res.render('index',
               { message: message.getViewMessage('index'),
                 data: JSON.parse(JSON.stringify(result)) });
  }).error(function(error) {
    res.contentType('application/json');
    var data = {
      result: false,
      error: message.getMessage('error',
                               'DatabaseError',
                               [JSON.stringify(error)])
    };
    var json = JSON.stringify(data);
    res.send(json);
  });
};


exports.tree = function(req, res) {
  var projectId = req.body.projectId;
  var tasks = [];
  // XVΏۂ̃vWFNg擾
  tasks.push(function(next) {
    db.Project.find(
      {
        include: [{
          model:db.RootPath
        }], where:[
          'id = ?',
          projectId
        ]
      }
    ).success(function(project) {
      next(null, project);
    }).error(function(error) {
      next(message.getMessage('error',
                               'DatabaseError',
                               [JSON.stringify(error)]));
    });
  });

  tasks.push(function(project, next) {
    var datas = [];
    for (var i = 0; i < project.rootPaths.length; ++i) {
      var data = {
        id: project.rootPaths[i].path,
        text: project.rootPaths[i].path,
        children: true,
        type: 'folder'
      };
      datas.push(data);
    }
    next(null, datas);
  });

  async.waterfall(tasks, function(err, data) {
    res.contentType('application/json');
    var json;
    if (err) {
      json = JSON.stringify({
        result: false,
        error: err
      });
    } else {
      json = JSON.stringify(data);
    }
    res.send(json);
  });
};


exports.tree_children = function(req, res) {
  var targetPath = req.body.id;

  fs.readdir(targetPath, function(err, files){
    res.contentType('application/json');

    if (err) {
      res.send(JSON.stringify([]));
      return;
    }
    var list = [];
    files.filter(function(file) {
      var fullfile = path.join(targetPath, file);
      return fullfile;
    }).forEach(function(file) {
      var fullfile = path.join(targetPath, file);
      var data = {};
      if (fs.statSync(fullfile).isDirectory()) {
        data.type = 'folder';
        data.children = true;
      } else {
        data.type = 'file';
        data.children = false;
      }
      data.id = fullfile;
      data.text = file;
      list.push(data);
    });

    res.send(JSON.stringify(list));
    return;    
  });
};


exports.runJasmine = function(req, res) {
  console.log('===============');
  var projectId = req.body.projectId;
  var rootTarget = req.body.id;
  var jasmineBin = path.join(__dirname, '../node_modules/jasmine-node/bin/jasmine-node');
  var timestamp = new Date() / 1000 | 0;
  var output = path.join(__dirname, '../output', projectId.toString(), timestamp.toString());
  var tasks = [];

  res.contentType('application/json');

  // ^CAEgh~
  var intervalId = setInterval(function() {
    res.write('\n');
  }, 1000);

  // o̓tH_̍쐬
  tasks.push(function(next) {
    console.log(output);
    mkdirp(output, function(err) {
      next(err);
    });
  });

  // Jasmine̎s
  tasks.push(function(next) {
    var script = util.format('%s --verbose --noColor --junitreport --output %s --captureExceptions %s', jasmineBin, output, rootTarget);
    exec(script, function(err, stdout, stderr) {
      console.log(stdout);
      console.log(stderr);
      next(stderr);
    });
  });


  // o͌ʂ̉
  tasks.push(function(next) {
    fs.readdir(output, function(err, files){
      if (err) {
        next(err);
        return;
      }
      var fullpaths = [];
      for (var i = 0; i < files.length; ++i) {
         fullpaths.push(path.join(output, files[i]));
      }
      readJunitXML(fullpaths, next);
    });
  });


  async.waterfall(tasks, function(err, results) {
    console.log('---------------------');
    console.log(err);
    console.log(util.inspect(results,false,null));
    clearInterval(intervalId);
    res.write(JSON.stringify(results));
    res.end();
  });
};


function readJunitXML(fullpaths, callback) {
  console.log('readJunitXML------------');
  var tasks = [];
  fullpaths.forEach(function(fullpath) {
    tasks.push(function(next) {
      fs.readFile(fullpath, 'utf8', function(err, text) {
        if (err) {
          next(err);
          return;
        }
        to_json(text, function(err, data) {
          next(err,
               {
                 name: path.basename(fullpath, path.extname(fullpath)),
                 data: convertResult(data)
               }
          );
        });
      });
    });
  });
  async.series(tasks, function(err, results) {
    callback(err, results);
  });


  function convertResult(json) {
    var results = [];
    if (!json.testsuites) {
      return results;
    }
    if (!json.testsuites.testsuite) {
      return results;
    }
    if (json.testsuites.testsuite.$) {
        results.push(convertTestSuite(json.testsuites.testsuite));
    } else {
      for (var key in json.testsuites.testsuite) {
        var testsuite = convertTestSuite(json.testsuites.testsuite[key]);
        results.push(testsuite);
      }
    }
    return results;
  }
  function convertTestSuite(xmlsuite) {
    var testsuite = {
      name: xmlsuite.$.name,
      errors: xmlsuite.$.errors,
      tests: xmlsuite.$.tests,
      failures: xmlsuite.$.failures,
      time: xmlsuite.$.time,
      timestamp: xmlsuite.$.timestamp,
      testcase:[]
    };
    if (xmlsuite.testcase) {
      if (xmlsuite.testcase.$) {
        testsuite.testcase.push(convertTestCase(xmlsuite.testcase));
      } else {
        for (var key in xmlsuite.testcase) {
          testsuite.testcase.push(convertTestCase(xmlsuite.testcase[key]));
        }
      }
    }
    return testsuite;
  }
  function convertTestCase(xmlcase) {
    return {
      name: xmlcase.$.name,
      time: xmlcase.$.time,
      failure: xmlcase.failure
    };
  }
}
