var util = require('util');
var message = require('./message.json');


/**
 * bZ[W̎擾
 * @param {string} category JeS
 * @param {string} id ID
 * @param {Array} args ̔z
 */
function _getMessage(category, id, args) {
  var msg = message.message[category];
  if (msg) {
    if (msg[id]) {
      var param = [msg[id]];
      if(args) {
        param = param.concat(args);
      }
      return util.format.apply(this, param);
    }
  }
  return util.format('category:%s id:%s args:%s' ,category, id, args);
}


/**
 * r[p̃bZ[W擾
 * @param {string} category JeS
 * @param {Array} args ̔z
 */
function _getViewMessage(category) {
  var msg = message.view[category];
  if (msg) {
    return msg;
  } else {
    return {};
  }
}


function _getFieldName(modelName, fieldName) {
  var ary = fieldName.split('.');
  var fname = ary[ary.length-1];
  var model = message.model[modelName];
  if (model) {
    return model[fname];
  }
  return '';
}


function _getValidationErrMsg(fieldValidate, validation) {
  var args = [];
  switch(validation) {
    case 'len':
      args = fieldValidate['len'];
      break;
  }
  return _getMessage('ValidationError', validation, args);
}


function _convertDbError(err, model) {
  var arys = [];
  if(err instanceof Array){
    var tmp = '';
    for (var i = 0; i < err.length ;++i) {
      tmp += _convertDbError(err[i], model) + '<BR>';
    }
    return tmp;
  }
  if (err.errno) {
    switch (err.errno) {
      case 5:
        return _getMessage('error',
                           'DbLockError');
      case 19:
        arys = err.message.split(': ');
        if (arys[1] === 'UNIQUE constraint failed') {
          var fields = arys[2].split(', ');
          var filename = '';
          for (var i = 0; i < fields.length; ++i) {
            if (i !== 0) {
              filename += ', ';
            }
            filename += _getFieldName(model.name, fields[i]);
          }

          return _getMessage('error',
                             'UniqueConstraintError',
                             [filename]);
        } else {
          return _getMessage('error', 'ConstraintError', [err.message]);
        }
      default:
        return _getMessage('error', 'DatabaseError', [err.code]);
    }
  } else {
    var result = '';
    for (var field in err) {
      result += _getMessage('error', 'FieldError', [_getFieldName(model.name, field)]);
      result += '<BR>\n';

      for(var i = 0; i < err[field].length; ++i) {
        arys = err[field][i].split(' ',2);
        if (arys.length != 2) {
          continue;
        }
        if (arys[0] != 'Validation') {
          continue;
        }
        result +=  _getValidationErrMsg(model.rawAttributes[field].validate, arys[1]);
        result += '<BR>\n';
      }
      result += '<BR>\n';
    }
    return result;
  }
}

module.exports = {
  getMessage: _getMessage,
  getViewMessage: _getViewMessage,
  convertDbError: _convertDbError
};
