/* GET home page. */
var db = require('../models');
var message = require('./message.js');
var async = require('async');
var dbUtil = require('./dbUtil.js');
var fs = require('fs');


/**
 * vWFNgꗗ
 * @param {Object} req NGXg
 * @param {Object} res 
 */
exports.index = function(req, res){
  db.Project.findAll({ include: [{
    model:db.RootPath
  }]}).success(function(result) {
    res.render('project',
               { message: message.getViewMessage('project'),
                 data: JSON.parse(JSON.stringify(result)) });
  }).error(function(error) {
    res.contentType('application/json');
    var data = {
      result: false,
      error: message.getMessage('error',
                               'DatabaseError',
                               [JSON.stringify(error)])
    };
    var json = JSON.stringify(data);
    res.send(json);
  });
};


function checkDirectory(paths, callback) {
  var tasks = [];
  var errstring = '';
  if (!(paths instanceof Array) || paths.length == 0) {
    errstring += message.getMessage('error',
                         'RequirementFieldError', 'path');
    callback(errstring);
    return;
  }

  paths.forEach(function(p) {
    tasks.push(function(next) {
      fs.stat(p, function(err, stats) {
        if (err) {
          errstring += message.getMessage('error',
                               'NotDirectoryError',
                               p) + '<BR>';
        }
        next(null);
      });
    });
  });
 
  async.waterfall(tasks, function(err) {
    callback(errstring);
  });
}


/**
 * vWFNg̒ǉ
 * @param {Object} req NGXg
 * @param {Object} res 
 */
exports.add = function(req, res) {
  var projectName = req.body.projectName;
  var paths = req.body.selectPath;

  var tasks = [];
  var trn = null;

  // pX̃`FbN
  tasks.push(function(t, next) {
    trn = t;
    checkDirectory(paths, next);
  });

  // vWFNg̒ǉ
  tasks.push(function(next) {
    var project = db.Project.build({
      name: projectName
    });
    project
      .save({transaction:trn})
      .complete(function(err) {
        var data = {result:true};
        if (err) {
          next(message.convertDbError(err, db.Project));
          return;
        }
        next(null, project);
      }
    );
  });

  // RootPath̍őID̎擾
  tasks.push(function(project, next) {
    db.RootPath.max('id', {transaction:trn}).success(function(maxid) {
      if (maxid == null) {
        maxid = 0;
      }
      next(null, project, maxid);
    }).error(function(error) {
      next(message.convertDbError(error, db.RootPath));
    });
  });

  // RootPath̒ǉƒǉACe̎擾
  tasks.push(function(project, maxid, next) {
    var datas = [];
    for (var i = 0; i < paths.length; ++i) {
      datas.push({
        path: paths[i],
        ProjectId: project.id
      });
    }
    db.RootPath.bulkCreate(datas, {transaction:trn, validate:true}).success(function(datas) {
      db.RootPath.findAll({ where : ['id > ?', maxid]}, {transaction:trn}).success(function(datas) {
        next(null, project, datas);
      }).error(function(error) {
        next(message.convertDbError(error, db.RootPath));
      });
    }).error(function(error) {
      next(message.convertDbError(error, db.RootPath));
    });
  });

  // ProjectRootPath̊֘AÂ
  tasks.push(function(project, paths, next) {
    project.setRootPaths(paths,{transaction:trn} ).success(function(datas) {
      next(null);
    }).error(function(error) {
      next(message.convertDbError(error, db.Project));
    });
  });

  //
  dbUtil.doTransaction(tasks, function(json) {
    res.contentType('application/json');
    res.send(json);
  });
};


/**
 * vWFNg̍폜
 * @param {Object} req NGXg
 * @param {Object} res 
 */
exports.delete = function(req, res) {
  var projectId = req.body.projectId;
  var paths = req.body.selectPath;

  var tasks = [];
  var trn = null;

  tasks.push(function(t, next) {
    trn = t;
    db.Project.find(
      {
        include: [{
          model:db.RootPath
        }], where:[
          'id = ?',
          projectId
        ]
      }
    ).success(function(project) {
      next(null, project);
    }).error(function(error) {
      next(message.getMessage('error',
                               'DatabaseError',
                               [JSON.stringify(error)]));
    });
  });

  // ProjectRootPath̊֘AÂ̍폜
  tasks.push(function(project, next) {
    project.setRootPaths(null,{transaction:trn}).success(function(datas) {
      next(null, project);
    }).error(function(error) {
      next(message.convertDbError(error, db.Project));
    });
  });

  // RootPaths̍폜
  tasks.push(function(project, next) {
    var where_string = 'id in (';
    var where = [];
    for (var i = 0 ; i < project.rootPaths.length; ++i) {
      if (i === 0) {
        where_string += '?';
      } else {
        where_string += ', ?';
      }
      where.push(project.rootPaths[i].id);
    }
    where_string += ')';
    where.splice(0, 0, where_string);

    db.RootPath.destroy(where, {transaction:trn}).success(function() {
      next(null, project);
    }).error(function(error) {
      next(message.convertDbError(error, db.RootPath));
    });
  });

  // Project̍폜
  tasks.push(function(project, next) {
    project.destroy({transaction:trn}).success(function() {
      next(null, project);
    }).error(function(error) {
      next(message.convertDbError(error, db.RootPath));
    });
  });

  //
  dbUtil.doTransaction(tasks, function(json) {
    res.contentType('application/json');
    res.send(json);
  });
};


/**
 * vWFNg̍XV
 * @param {Object} req NGXg
 * @param {Object} res 
 */
exports.update = function(req, res) {
  var projectId = req.body.projectId;
  var projectName = req.body.projectName;
  var paths = req.body.selectPath;
  var tasks = [];
  var trn = null;  // pX̃`FbN
  tasks.push(function(t, next) {
    trn = t;
    checkDirectory(paths, next);
  });

  // XVΏۂ̃vWFNg擾
  tasks.push(function(next) {
    db.Project.find(
      {
        include: [{
          model:db.RootPath
        }], where:[
          'id = ?',
          projectId
        ]
      }
    ).success(function(project) {
      next(null, project);
    }).error(function(error) {
      next(message.getMessage('error',
                               'DatabaseError',
                               [JSON.stringify(error)]));
    });
  });

  // Project̖̂̍XV
  tasks.push(function(project, next) {
    project.name = projectName;

    project
      .save({transaction:trn})
      .complete(function(err) {
        var data = {result:true};
        if (err) {
          next(message.convertDbError(err, db.Project));
          return;
        }
        next(null, project);
      }
    );
  });

  // ProjectRootPath̊֘AÂ̍폜
  tasks.push(function(project, next) {
    project.setRootPaths(null,{transaction:trn}).success(function(datas) {
      next(null, project);
    }).error(function(error) {
      next(message.convertDbError(error, db.Project));
    });
  });

  // RootPaths̍폜
  tasks.push(function(project, next) {
    var where_string = 'id in (';
    var where = [];
    for (var i = 0 ; i < project.rootPaths.length; ++i) {
      if (i === 0) {
        where_string += '?';
      } else {
        where_string += ', ?';
      }
      where.push(project.rootPaths[i].id);
    }
    where_string += ')';
    where.splice(0, 0, where_string);

    db.RootPath.destroy(where, {transaction:trn}).success(function() {
      next(null, project);
    }).error(function(error) {
      next(message.convertDbError(error, db.RootPath));
    });
  });

  // RootPath̍őID̎擾
  tasks.push(function(project, next) {
    db.RootPath.max('id', {transaction:trn}).success(function(maxid) {
      if (maxid == null) {
        maxid = 0;
      }
      next(null, project, maxid);
    }).error(function(error) {
      next(message.convertDbError(error, db.RootPath));
    });
  });

  // RootPath̒ǉƒǉACe̎擾
  tasks.push(function(project, maxid, next) {
    var datas = [];
    for (var i = 0; i < paths.length; ++i) {
      datas.push({
        path: paths[i],
        ProjectId: project.id
      });
    }
    db.RootPath.bulkCreate(datas, {transaction:trn, validate:true}).success(function(datas) {
      db.RootPath.findAll({ where : ['id > ?', maxid]}, {transaction:trn}).success(function(datas) {
        next(null, project, datas);
      }).error(function(error) {
        next(message.convertDbError(error, db.RootPath));
      });
    }).error(function(error) {
      next(message.convertDbError(error, db.RootPath));
    });
  });

  // ProjectRootPath̊֘AÂ
  tasks.push(function(project, paths, next) {
    project.setRootPaths(paths,{transaction:trn} ).success(function(datas) {
      next(null);
    }).error(function(error) {
      next(message.convertDbError(error, db.Project));
    });
  });
 
  //
  dbUtil.doTransaction(tasks, function(json) {
    res.contentType('application/json');
    res.send(json);
  });
};